@@ -83,7 +83,7 @@ class DataJunction(object):
     
     It handles BYPASS_VALS constants to pass them to the output
     (only one if there are many). """
-    def __init__(self, joiner=None, strip_bypass=True):
+    def __init__(self, joiner=None, strip_bypass=True, node=None):
         self.input_sources = list()
         self.joiner = joiner or linear_merging
         self.runner = None
@@ -93,6 +93,7 @@ class DataJunction(object):
             self.strip_bypass = strip_bypass
             
         self.queue = collections.deque()
+        self.node = node
         
     def add_input_source(self, source, name):
         """ Appends a named input source """
@@ -136,9 +137,17 @@ class DataJunction(object):
         Warning, this function shouldn't be called twice !
         """
         if self.runner is None:
+            extra_args = {'ports' : map(itemgetter(1),
+                                        self.input_sources)}
+            func_spec = inspect.getargspec(self.joiner)
+            func_kwargs = func_spec[0][len(func_spec[3])*-1:]
+            if "metadata" in func_kwargs:
+                extra_args['metadata'] = \
+                    {'get_buffer_state':self.node.get_buffer_state,
+                     'get_input_buffer':self.node.get_input_buffer}
+                
             self.runner = runner(self.joiner,
-                                 {'ports' : map(itemgetter(1),
-                                                self.input_sources)},
+                                 extra_args,
                                  handle_bypass=False)
         
         for input_source, name in self.input_sources:
@@ -316,7 +325,8 @@ class Node(object):
                 
             if len(self.input_ports) > 1:
                 self.junction = DataJunction(self.joiner,
-                                             strip_bypass=self.joiner_strip_bypass)
+                                             strip_bypass=self.joiner_strip_bypass,
+                                             node=self)
         
         if len(self.output_ports) > 1:
             if self.splitter:
@@ -429,18 +439,25 @@ class Node(object):
     def get_output_port_position(self, generator):
         return self.output_generators.index(generator)
         
-    def get_input_buffer(self):
+    def get_input_buffer(self, recursive=True, separate=False):
         if not self.is_producer:
             if len(self.input_ports) > 1:
                 in_buffers = list()
                 # TODO IMPLEMENT
                 for i, up_node in enumerate(self.input_ports):
                     source, name = self.junction.input_sources[i]
-                    in_buffers.append(up_node.get_buffer_state(source))
-                return sum(in_buffers)
+                    in_buffers.append((name, up_node.get_buffer_state(source)))
+                
+                if separate:
+                    return dict(in_buffers)
+                
+                else:
+                    return sum(map(itemgetter(1), in_buffers))
+                
             else:
                 up_node = self.input_ports[0]
-                return up_node.get_buffer_state(self.input_flow)
+                return up_node.get_buffer_state(self.input_flow,
+                                                recursive=recursive)
         
         return 0
         
@@ -451,7 +468,7 @@ class Node(object):
         
         return list()
     
-    def get_buffer_state(self, flow):
+    def get_buffer_state(self, flow, recursive=True):
         position = self.get_output_port_position(flow)
         if not self.is_consumer and len(self.output_ports) > 1:
             if 'out' in self.splitter.out_array_lookup:
@@ -460,9 +477,10 @@ class Node(object):
             else:
                 # not initialized yet...
                 return 0
-        else:
+        elif recursive:
             return self.get_input_buffer()
-            
+        else:
+            return 0
 
 class Network(object):
     """ Initialises a network of nodes from a dictionary structure.
@@ -547,7 +565,7 @@ class Network(object):
         for node in self.consumer_nodes:
             node.link_up()
             
-    def get_status_handler(self, advanced=True):
+    def get_status_handler(self, advanced=False):
         """ Returns a status handler that returns True
         each time it receives a True from parents.
         
@@ -580,7 +598,8 @@ class Network(object):
             final_consumer = runner(all_true_longest)
             for num, consumer in enumerate(self.consumers):
                 final_consumer.connect_in("sources", consumer)
-                
+            
             return final_consumer('out')
+#            return post_final(final_consumer('out'))
 #            for info in izip_longest(fillvalue=True, *self.consumers):   
 #                yield reduce(operator.and_, map(bool, info))
